import fetch from 'node-fetch'
import crypto from 'crypto'

class FileCache {
  constructor () {
    this._cache = {}
  }

  async warmUp (manifests, deps) {
    const cache = Object.fromEntries(await (async function () {
      const files = Object.keys(deps)
      const chunkSize = Math.ceil(files.length / 50)
      const result = []
      while (files.length > 0) {
        result.push.apply(result, (await Promise.all(files.splice(0, chunkSize).map(async file => {
          try {
            const manifest = manifests[file] || Object.values(manifests).find(m =>
              m.file === file ||
              (m?.assets?.indexOf(file) >= 0) ||
              (m?.css?.indexOf(file) >= 0)
            )
            if (!manifest) {
              console.error('could not find manifest for', file)
              return null
            }
            const response = await fetch(new URL(file, manifest.meta.baseUrl))
            if (!response.ok) return null
            const content = await response.buffer()
            const sha256Sum = crypto.createHash('sha256').update(content).digest('base64')
            return [file, {
              'content-type': response.headers.get('content-type'),
              sha256Sum,
              content
            }]
          } catch (e) { console.error(e) }
        }))).filter(data => Array.isArray(data) && data.length === 2))
      }
      return result
    }()))
    this._cache = cache
  }

  get (path) {
    return this?._cache[path.slice(1)] || {}
  }
}

export const fileCache = new FileCache()
